import { DOCUMENT, CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, Directive, Optional, Inject, Injectable, Component, Input, NgModule, makeEnvironmentProviders } from '@angular/core';
import * as i1 from '@angular/cdk/platform';
import { coerceElement, coerceBooleanProperty } from '@angular/cdk/coercion';

const NGX_CONFETTI_GLOBAL_CONFIG = new InjectionToken('ngx-confetti-global-config');
const defaultColors = ["#a864fd", "#29cdff", "#78ff44", "#ff718d", "#fdff6a"];
const defaultsConfettiConfig = {
    angle: 90,
    spread: 45,
    startVelocity: 45,
    elementCount: 50,
    width: "10px",
    height: "10px",
    perspective: "",
    colors: defaultColors,
    duration: 3000,
    stagger: 0,
    dragFriction: 0.1,
    random: Math.random
};

class ConfettiRenderer {
    constructor(_document) {
        this._document = _document;
    }
    createElements(elementCount, colors, width, height) {
        const fragment = this._document.createDocumentFragment();
        const elements = Array.from({ length: elementCount }).map((_, index) => {
            const element = this._document.createElement('div');
            const color = colors[index % colors.length];
            element.style.backgroundColor = color;
            element.style.width = width;
            element.style.height = height;
            element.style.position = 'absolute';
            element.style.willChange = 'transform, opacity';
            element.style.visibility = 'hidden';
            fragment.appendChild(element);
            return element;
        });
        return {
            fragment,
            elements,
        };
    }
    randomPhysics(angle, spread, startVelocity, random) {
        const radAngle = angle * (Math.PI / 180);
        const radSpread = spread * (Math.PI / 180);
        return {
            x: 0,
            y: 0,
            z: 0,
            wobble: random() * 10,
            wobbleSpeed: 0.1 + random() * 0.1,
            velocity: startVelocity * 0.5 + random() * startVelocity,
            angle2D: -radAngle + (0.5 * radSpread - random() * radSpread),
            angle3D: -(Math.PI / 4) + random() * (Math.PI / 2),
            tiltAngle: random() * Math.PI,
            tiltAngleSpeed: 0.1 + random() * 0.3,
        };
    }
    updateFetti(fetti, progress, dragFriction, decay) {
        fetti.physics.x += Math.cos(fetti.physics.angle2D) * fetti.physics.velocity;
        fetti.physics.y += Math.sin(fetti.physics.angle2D) * fetti.physics.velocity;
        fetti.physics.z += Math.sin(fetti.physics.angle3D) * fetti.physics.velocity;
        fetti.physics.wobble += fetti.physics.wobbleSpeed;
        // Backward compatibility
        if (decay) {
            fetti.physics.velocity *= decay;
        }
        else {
            fetti.physics.velocity -= fetti.physics.velocity * dragFriction;
        }
        fetti.physics.y += 3;
        fetti.physics.tiltAngle += fetti.physics.tiltAngleSpeed;
        const { x, y, z, tiltAngle, wobble } = fetti.physics;
        const wobbleX = x + 10 * Math.cos(wobble);
        const wobbleY = y + 10 * Math.sin(wobble);
        const transform = `translate3d(${wobbleX}px, ${wobbleY}px, ${z}px) rotate3d(1, 1, 1, ${tiltAngle}rad)`;
        fetti.element.style.visibility = "visible";
        fetti.element.style.transform = transform;
        fetti.element.style.opacity = (1 - progress).toString();
        ;
    }
    animate(root, fettis, dragFriction, decay, duration, stagger) {
        let startTime;
        let isActive = true;
        const update = (time) => {
            if (!isActive)
                return;
            if (!startTime)
                startTime = time;
            const elapsed = time - startTime;
            const progress = startTime === time ? 0 : (time - startTime) / duration;
            fettis.slice(0, Math.ceil(elapsed / stagger)).forEach(fetti => {
                this.updateFetti(fetti, progress, dragFriction, decay);
            });
            if (time - startTime < duration) {
                requestAnimationFrame(update);
            }
            else {
                fettis.forEach((fetti) => {
                    if (fetti.element.parentNode === root) {
                        root.removeChild(fetti.element);
                    }
                });
                isActive = false;
            }
        };
        requestAnimationFrame(update);
        return new Promise(resolve => {
            const checkIfDone = setInterval(() => {
                if (!isActive) {
                    clearInterval(checkIfDone);
                    resolve();
                }
            }, 100);
        });
    }
    launch(root, config) {
        const { elementCount, colors, width, height, perspective, angle, spread, startVelocity, decay, dragFriction, duration, stagger, random, } = { ...defaultsConfettiConfig, ...config };
        root.style.perspective = perspective;
        const { fragment, elements } = this.createElements(elementCount ?? 50, colors, width, height);
        root.appendChild(fragment);
        const fettis = elements.map((element) => ({
            element,
            physics: this.randomPhysics(angle, spread, startVelocity, random),
        }));
        return this.animate(root, fettis, dragFriction, decay, duration, stagger);
    }
}

class NgxConfettiDirective {
    constructor(_elementRef, ngZone, platform, document, globalConfig) {
        this._elementRef = _elementRef;
        this.ngZone = ngZone;
        this.platform = platform;
        this._globalConfig = (globalConfig ||
            defaultsConfettiConfig);
        this._confettiRenderer = new ConfettiRenderer(document);
    }
    launch(config) {
        if (!this.platform.isBrowser) {
            return;
        }
        // const element = coerceElement(this._elementRef);
        this.ngZone.runOutsideAngular(() => {
            this._confettiRenderer.launch(this._elementRef.nativeElement, config ?? this._globalConfig);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxConfettiDirective, deps: [{ token: i0.ElementRef }, { token: i0.NgZone }, { token: i1.Platform }, { token: DOCUMENT, optional: true }, { token: NGX_CONFETTI_GLOBAL_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.5", type: NgxConfettiDirective, isStandalone: true, selector: "[confetti], [ngxConfetti]", exportAs: ["ngxConfetti"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxConfettiDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[confetti], [ngxConfetti]',
                    exportAs: 'ngxConfetti',
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i0.NgZone }, { type: i1.Platform }, { type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [NGX_CONFETTI_GLOBAL_CONFIG]
                }] }] });

class NgxDomConfettiService {
    constructor(document, globalConfig) {
        this._confettiRenderer = new ConfettiRenderer(document);
        this._globalConfig = globalConfig;
    }
    open(el, config) {
        this._confettiRenderer.launch(coerceElement(el), config ?? this._globalConfig);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiService, deps: [{ token: DOCUMENT, optional: true }, { token: NGX_CONFETTI_GLOBAL_CONFIG }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [NGX_CONFETTI_GLOBAL_CONFIG]
                }] }] });

class NgxDomConfettiComponent {
    set config(_config) {
        if (_config) {
            this._config = Object.assign(this._config, _config);
        }
    }
    set active(_active) {
        this._active = coerceBooleanProperty(_active);
        if (this._active) {
            this._confettiRenderer.launch(coerceElement(this.el), this._config);
        }
    }
    constructor(el, document, globalConfig) {
        this.el = el;
        this._config = {
            ...globalConfig,
        };
        this._confettiRenderer = new ConfettiRenderer(document);
        this._active = false;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiComponent, deps: [{ token: i0.ElementRef }, { token: DOCUMENT, optional: true }, { token: NGX_CONFETTI_GLOBAL_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.0.5", type: NgxDomConfettiComponent, isStandalone: true, selector: "ngx-dom-confetti", inputs: { config: "config", active: "active" }, ngImport: i0, template: ` <ng-content></ng-content> `, isInline: true }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ngx-dom-confetti', template: ` <ng-content></ng-content> ` }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [NGX_CONFETTI_GLOBAL_CONFIG]
                }] }], propDecorators: { config: [{
                type: Input
            }], active: [{
                type: Input
            }] } });

class NgxDomConfettiModule {
    static forRoot() {
        return {
            ngModule: NgxDomConfettiModule,
            providers: [
                {
                    provide: NGX_CONFETTI_GLOBAL_CONFIG,
                    useValue: defaultsConfettiConfig,
                },
                NgxDomConfettiService,
            ],
        };
    }
    static forChild() {
        return {
            ngModule: NgxDomConfettiModule,
            providers: [
                {
                    provide: NGX_CONFETTI_GLOBAL_CONFIG,
                    useValue: defaultsConfettiConfig,
                },
                NgxDomConfettiService,
            ],
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiModule, imports: [CommonModule, NgxDomConfettiComponent, NgxConfettiDirective], exports: [NgxDomConfettiComponent, NgxConfettiDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiModule, providers: [
            {
                provide: NGX_CONFETTI_GLOBAL_CONFIG,
                useValue: defaultsConfettiConfig,
            },
            NgxDomConfettiService,
        ], imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: NgxDomConfettiModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [CommonModule, NgxDomConfettiComponent, NgxConfettiDirective],
                    providers: [
                        {
                            provide: NGX_CONFETTI_GLOBAL_CONFIG,
                            useValue: defaultsConfettiConfig,
                        },
                        NgxDomConfettiService,
                    ],
                    exports: [NgxDomConfettiComponent, NgxConfettiDirective],
                }]
        }] });

function provideConfetti(config) {
    return makeEnvironmentProviders([
        {
            provide: NGX_CONFETTI_GLOBAL_CONFIG,
            useValue: config ?? defaultsConfettiConfig,
        },
    ]);
}

/*
 * Public API Surface of ngx-dom-confetti
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_CONFETTI_GLOBAL_CONFIG, NgxConfettiDirective, NgxDomConfettiComponent, NgxDomConfettiModule, NgxDomConfettiService, defaultsConfettiConfig, provideConfetti };
//# sourceMappingURL=ngx-dom-confetti.mjs.map
